#include <io.h>
#include <signal.h>
#include <interrupt.h>
#include <timer.h>
#include <eeprom.h>
#include <wdt.h>
#include "global.h"
#include "lcd.h"
#include "ds1302.h"
#include "dcf77.h"

u08 prescale;
u08 update;

struct dt act_date;
struct dt dcf_date;

SIGNAL(SIG_OVERFLOW0)        /* signal handler for tcnt0 overflow interrupt */
{
	prescale++;
	dcf77_timetick();

	if(prescale == TICK_100M)
	{
		prescale = 0;
		update = 1;
	}
}


int main(void)
{
	u08 led;
	u08 temp;

	cbi(DDRB,PB1); 					// Contrast Pin, switch to input
	cbi(PORTB,PB1);					// Contrast Pin, switch to high impedance
	outp(0x80,ACSR);  				// analog comparator switched off
	outp(0x40,GIMSK);					// Enable Int 0 for DCF77 Receiver
	outp(0x02,MCUCR);					// Int 0 falling Edge

	cbi(DDRD,PD2);						// switch DCF77 Port to input
	cbi(PORTD,PD2);					// high Impedance

	outp(CK8,TCCR0);					// counter prescaler CK/8
	outp(1,TCNT0); 					// set up timer0 -> 125 INTs/s
	outp(0xAA ,TIMSK); 				// enable timer0 int

	init_ds1302();
	dcf77_init();
	lcd_init();

	prescale		= 0;
	led			= 0;
	update 		= 0;

	read_time_ds1302(&act_date);

	lcd_clear();
	lcd_mode(LCD_ON_CURSOR_OFF);

//	wdt_reset();
//	wdt_enable(0x07);

	sei();

	for(;;)    /* loop forever */
	{
		if(dcf77_update)					// new DCF-Bit received
		{
			dcf77_update = 0;
			dcf77_process_data();
		}

		if(update_time)					// DCF Time Information complete, update display Time and Timekeeper
		{
			update_time 		= 0;
			dcf77_tick 			= 0;
			act_date.t.s 		= dcf_date.t.s;
			act_date.t.m 		= dcf_date.t.m;
			act_date.t.h 		= dcf_date.t.h;
			act_date.d.tag 	= dcf_date.d.tag;
			act_date.d.wt 		= dcf_date.d.wt;
			act_date.d.monat 	= dcf_date.d.monat;
			act_date.d.jahr 	= dcf_date.d.jahr;

			write_time_ds1302(&act_date);
		}

		if(inc_time)						// Inrement Display Time, 1 second Intervall
		{
			inc_time = 0;
			increment_time(&act_date);
			display_date(1,&act_date);
		}

		if(update)							// 100ms Intervall
		{
//			wdt_reset();					// Watchdog Reset
			update = 0;
	
			if(led == 0)							// alive LED
			{
				cbi(PORTB,PB5);
				led = 1;
			}
			else
			{
				sbi(PORTB,PB5);
				led = 0;
			}
		}
	}
}
